# Generated by roxygen2: do not edit by hand

S3method("[",msts)
S3method(accuracy,Arima)
S3method(accuracy,fc_model)
S3method(accuracy,forecast)
S3method(accuracy,lm)
S3method(accuracy,mforecast)
S3method(accuracy,numeric)
S3method(accuracy,ts)
S3method(as.Date,timeDate)
S3method(as.character,Arima)
S3method(as.character,bats)
S3method(as.character,ets)
S3method(as.character,tbats)
S3method(as.data.frame,forecast)
S3method(as.data.frame,mforecast)
S3method(as.ts,forecast)
S3method(autolayer,forecast)
S3method(autolayer,mforecast)
S3method(autolayer,msts)
S3method(autolayer,mts)
S3method(autolayer,ts)
S3method(autoplot,Arima)
S3method(autoplot,StructTS)
S3method(autoplot,acf)
S3method(autoplot,ar)
S3method(autoplot,bats)
S3method(autoplot,decomposed.ts)
S3method(autoplot,ets)
S3method(autoplot,forecast)
S3method(autoplot,mforecast)
S3method(autoplot,mpacf)
S3method(autoplot,mstl)
S3method(autoplot,msts)
S3method(autoplot,mts)
S3method(autoplot,seas)
S3method(autoplot,splineforecast)
S3method(autoplot,stl)
S3method(autoplot,tbats)
S3method(autoplot,ts)
S3method(coef,ets)
S3method(fitted,ARFIMA)
S3method(fitted,Arima)
S3method(fitted,ar)
S3method(fitted,bats)
S3method(fitted,ets)
S3method(fitted,forecast_ARIMA)
S3method(fitted,modelAR)
S3method(fitted,nnetar)
S3method(fitted,rw_model)
S3method(fitted,tbats)
S3method(fitted,tslm)
S3method(forecast,Arima)
S3method(forecast,HoltWinters)
S3method(forecast,StructTS)
S3method(forecast,ar)
S3method(forecast,baggedModel)
S3method(forecast,bats)
S3method(forecast,croston_model)
S3method(forecast,default)
S3method(forecast,ets)
S3method(forecast,forecast)
S3method(forecast,forecast_ARIMA)
S3method(forecast,fracdiff)
S3method(forecast,lm)
S3method(forecast,mean_model)
S3method(forecast,mlm)
S3method(forecast,modelAR)
S3method(forecast,mstl)
S3method(forecast,mts)
S3method(forecast,nnetar)
S3method(forecast,rw_model)
S3method(forecast,spline_model)
S3method(forecast,stl)
S3method(forecast,stlm)
S3method(forecast,tbats)
S3method(forecast,theta_model)
S3method(forecast,ts)
S3method(forecast,varest)
S3method(fortify,ts)
S3method(getResponse,Arima)
S3method(getResponse,ar)
S3method(getResponse,baggedModel)
S3method(getResponse,bats)
S3method(getResponse,default)
S3method(getResponse,fracdiff)
S3method(getResponse,lm)
S3method(getResponse,mforecast)
S3method(getResponse,tbats)
S3method(hfitted,Arima)
S3method(hfitted,default)
S3method(hfitted,ets)
S3method(logLik,ets)
S3method(modeldf,Arima)
S3method(modeldf,bats)
S3method(modeldf,default)
S3method(modeldf,ets)
S3method(modeldf,lm)
S3method(modeldf,meanf)
S3method(modeldf,rw_model)
S3method(nobs,ets)
S3method(plot,Arima)
S3method(plot,ar)
S3method(plot,armaroots)
S3method(plot,bats)
S3method(plot,ets)
S3method(plot,forecast)
S3method(plot,mforecast)
S3method(plot,mpacf)
S3method(plot,splineforecast)
S3method(plot,tbats)
S3method(predict,default)
S3method(print,CVar)
S3method(print,OCSBtest)
S3method(print,baggedModel)
S3method(print,bats)
S3method(print,croston_model)
S3method(print,ets)
S3method(print,forecast)
S3method(print,forecast_ARIMA)
S3method(print,mean_model)
S3method(print,mforecast)
S3method(print,modelAR)
S3method(print,msts)
S3method(print,nnetar)
S3method(print,nnetarmodels)
S3method(print,rw_model)
S3method(print,spline_model)
S3method(print,summary.Arima)
S3method(print,summary.ets)
S3method(print,summary.forecast)
S3method(print,summary.mforecast)
S3method(print,tbats)
S3method(print,theta_model)
S3method(residuals,ARFIMA)
S3method(residuals,Arima)
S3method(residuals,ar)
S3method(residuals,bats)
S3method(residuals,ets)
S3method(residuals,forecast)
S3method(residuals,forecast_ARIMA)
S3method(residuals,modelAR)
S3method(residuals,nnetar)
S3method(residuals,spline_model)
S3method(residuals,stlm)
S3method(residuals,tbats)
S3method(residuals,tslm)
S3method(scale,ts)
S3method(seasadj,decomposed.ts)
S3method(seasadj,mstl)
S3method(seasadj,seas)
S3method(seasadj,stl)
S3method(seasadj,tbats)
S3method(simulate,Arima)
S3method(simulate,ar)
S3method(simulate,ets)
S3method(simulate,fracdiff)
S3method(simulate,modelAR)
S3method(simulate,nnetar)
S3method(simulate,rw_model)
S3method(simulate,spline_model)
S3method(simulate,tbats)
S3method(subset,forecast)
S3method(subset,msts)
S3method(subset,ts)
S3method(summary,Arima)
S3method(summary,ets)
S3method(summary,forecast)
S3method(summary,mforecast)
S3method(summary,tslm)
S3method(window,msts)
export("%>%")
export(Acf)
export(Arima)
export(BoxCox)
export(BoxCox.lambda)
export(CV)
export(CVar)
export(Ccf)
export(GeomForecast)
export(InvBoxCox)
export(Pacf)
export(StatForecast)
export(accuracy)
export(arfima)
export(arima.errors)
export(arimaorder)
export(auto.arima)
export(autolayer)
export(autoplot)
export(baggedETS)
export(baggedModel)
export(bats)
export(bizdays)
export(bld.mbb.bootstrap)
export(checkresiduals)
export(croston)
export(croston_model)
export(dm.test)
export(dshw)
export(easter)
export(ets)
export(findfrequency)
export(forecast)
export(forecast.ets)
export(fourier)
export(fourierf)
export(geom_forecast)
export(getResponse)
export(ggAcf)
export(ggCcf)
export(ggPacf)
export(gghistogram)
export(gglagchull)
export(gglagplot)
export(ggmonthplot)
export(ggseasonplot)
export(ggsubseriesplot)
export(ggtaperedacf)
export(ggtaperedpacf)
export(ggtsdisplay)
export(holt)
export(hw)
export(is.Arima)
export(is.acf)
export(is.baggedModel)
export(is.bats)
export(is.constant)
export(is.ets)
export(is.forecast)
export(is.mforecast)
export(is.modelAR)
export(is.nnetar)
export(is.nnetarmodels)
export(is.splineforecast)
export(is.stlm)
export(ma)
export(mean_model)
export(meanf)
export(modelAR)
export(modeldf)
export(monthdays)
export(mstl)
export(msts)
export(na.interp)
export(naive)
export(ndiffs)
export(nnetar)
export(nsdiffs)
export(ocsb.test)
export(remainder)
export(rw_model)
export(rwf)
export(seasadj)
export(seasonal)
export(seasonaldummy)
export(seasonaldummyf)
export(seasonplot)
export(ses)
export(sindexf)
export(snaive)
export(spline_model)
export(splinef)
export(stlf)
export(stlm)
export(taperedacf)
export(taperedpacf)
export(tbats)
export(tbats.components)
export(theta_model)
export(thetaf)
export(trendcycle)
export(tsCV)
export(tsclean)
export(tsdisplay)
export(tslm)
export(tsoutliers)
if (getRversion() < "4.5.0") S3method(head, ts)
if (getRversion() < "4.5.0") S3method(tail, ts)
import(Rcpp)
import(parallel)
importFrom(colorspace,sequential_hcl)
importFrom(fracdiff,diffseries)
importFrom(fracdiff,fracdiff)
importFrom(fracdiff,fracdiff.sim)
importFrom(generics,accuracy)
importFrom(generics,forecast)
importFrom(ggplot2,autolayer)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,fortify)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,nclass.FD)
importFrom(grDevices,palette)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(magrittr,"%>%")
importFrom(stats,"tsp<-")
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,Box.test)
importFrom(stats,acf)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,ar)
importFrom(stats,arima)
importFrom(stats,arima.sim)
importFrom(stats,as.formula)
importFrom(stats,as.ts)
importFrom(stats,complete.cases)
importFrom(stats,cycle)
importFrom(stats,decompose)
importFrom(stats,diffinv)
importFrom(stats,end)
importFrom(stats,extractAIC)
importFrom(stats,filter)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,frequency)
importFrom(stats,hatvalues)
importFrom(stats,is.mts)
importFrom(stats,is.ts)
importFrom(stats,ksmooth)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,logLik)
importFrom(stats,lsfit)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.contiguous)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pf)
importFrom(stats,plot.ts)
importFrom(stats,poly)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,simulate)
importFrom(stats,smooth.spline)
importFrom(stats,spec.ar)
importFrom(stats,start)
importFrom(stats,stl)
importFrom(stats,supsmu)
importFrom(stats,terms)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,tsdiag)
importFrom(stats,tsp)
importFrom(stats,var)
importFrom(stats,window)
importFrom(timeDate,Easter)
importFrom(timeDate,as.Date.timeDate)
importFrom(timeDate,as.timeDate)
importFrom(timeDate,difftimeDate)
importFrom(timeDate,isBizday)
importFrom(urca,ur.df)
importFrom(urca,ur.kpss)
importFrom(urca,ur.pp)
importFrom(utils,head)
importFrom(utils,head.matrix)
importFrom(utils,packageVersion)
importFrom(utils,tail)
importFrom(utils,tail.matrix)
importFrom(zoo,as.Date)
importFrom(zoo,as.yearqtr)
importFrom(zoo,rollmean)
useDynLib(forecast, .registration = TRUE)
