#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

#' @importFrom generics forecast
#' @export
generics::forecast

#' @importFrom generics accuracy
#' @export
generics::accuracy

#' @importFrom ggplot2 autoplot
#' @export
ggplot2::autoplot

#' @importFrom ggplot2 autolayer
#' @export
ggplot2::autolayer
