# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calcBATS <- function(y, yHat, wTranspose, F, x, g, e) {
    .Call(`_forecast_calcBATS`, y, yHat, wTranspose, F, x, g, e)
}

calcBATSFaster <- function(y, yHat, wTranspose, F, x, g, e, xNought, sPeriods, beta, tau, p, q) {
    invisible(.Call(`_forecast_calcBATSFaster`, y, yHat, wTranspose, F, x, g, e, xNought, sPeriods, beta, tau, p, q))
}

calcWTilda <- function(wTildaTranspose, D) {
    .Call(`_forecast_calcWTilda`, wTildaTranspose, D)
}

calcTBATSFaster <- function(y, yHat, wTranspose, F, x, g, e, xNought) {
    invisible(.Call(`_forecast_calcTBATSFaster`, y, yHat, wTranspose, F, x, g, e, xNought))
}

etsTargetFunctionInit <- function(p_y, p_nstate, p_errortype, p_trendtype, p_seasontype, p_damped, p_lower, p_upper, p_opt_crit, p_nmse, p_bounds, p_m, p_optAlpha, p_optBeta, p_optGamma, p_optPhi, p_givenAlpha, p_givenBeta, p_givenGamma, p_givenPhi, p_alpha, p_beta, p_gamma, p_phi, p_rho) {
    .Call(`_forecast_etsTargetFunctionInit`, p_y, p_nstate, p_errortype, p_trendtype, p_seasontype, p_damped, p_lower, p_upper, p_opt_crit, p_nmse, p_bounds, p_m, p_optAlpha, p_optBeta, p_optGamma, p_optPhi, p_givenAlpha, p_givenBeta, p_givenGamma, p_givenPhi, p_alpha, p_beta, p_gamma, p_phi, p_rho)
}

etsNelderMead <- function(p_var, p_env, p_abstol, p_intol, p_alpha, p_beta, p_gamma, p_trace, p_maxit) {
    .Call(`_forecast_etsNelderMead`, p_var, p_env, p_abstol, p_intol, p_alpha, p_beta, p_gamma, p_trace, p_maxit)
}

makeBATSWMatrix <- function(smallPhi, sPeriods, arCoefs, maCoefs) {
    .Call(`_forecast_makeBATSWMatrix`, smallPhi, sPeriods, arCoefs, maCoefs)
}

makeBATSGMatrix <- function(alpha, beta, gammaVector, seasonalPeriods, p, q) {
    .Call(`_forecast_makeBATSGMatrix`, alpha, beta, gammaVector, seasonalPeriods, p, q)
}

makeTBATSWMatrix <- function(smallPhi, kVector, arCoefs, maCoefs, tau) {
    .Call(`_forecast_makeTBATSWMatrix`, smallPhi, kVector, arCoefs, maCoefs, tau)
}

makeCIMatrix <- function(k, m) {
    .Call(`_forecast_makeCIMatrix`, k, m)
}

makeSIMatrix <- function(k, m) {
    .Call(`_forecast_makeSIMatrix`, k, m)
}

makeAIMatrix <- function(C, S, k) {
    .Call(`_forecast_makeAIMatrix`, C, S, k)
}

updateFMatrix <- function(F, smallPhi, alpha, beta, gammaBold, ar, ma, tau) {
    invisible(.Call(`_forecast_updateFMatrix`, F, smallPhi, alpha, beta, gammaBold, ar, ma, tau))
}

updateWtransposeMatrix <- function(wTranspose, smallPhi, tau, arCoefs, maCoefs, p, q) {
    invisible(.Call(`_forecast_updateWtransposeMatrix`, wTranspose, smallPhi, tau, arCoefs, maCoefs, p, q))
}

updateGMatrix <- function(g, gammaBold, alpha, beta, gammaVector, seasonalPeriods) {
    invisible(.Call(`_forecast_updateGMatrix`, g, gammaBold, alpha, beta, gammaVector, seasonalPeriods))
}

updateTBATSGammaBold <- function(gammaBold, kVector, gammaOne, gammaTwo) {
    invisible(.Call(`_forecast_updateTBATSGammaBold`, gammaBold, kVector, gammaOne, gammaTwo))
}

updateTBATSGMatrix <- function(g, gammaBold, alpha, beta) {
    invisible(.Call(`_forecast_updateTBATSGMatrix`, g, gammaBold, alpha, beta))
}

